@@ -28,11 +28,11 @@ using namespace UCOMMON_NAMESPACE;
 
 int main(int argc, char **argv)
 {
-	const char *proto = "http";
-	secure::context_t ctx = NULL;
+	const char *proto = "80";
+	secure::client_t ctx = NULL;
 
 	if(secure::init()) {
-		proto = "https";
+		proto = "443";
 		ctx = secure::client();
 	}
 
@@ -41,11 +41,11 @@ int main(int argc, char **argv)
 	if(ctx)
 	    printf("ctx %p %d\n", (void *)ctx, ctx->err());
 
-    ssl_t ssl(proto, ctx);
-    ssl.open("www.google.com");
-    printf("open %d\n", ssl.isopen());
+    ssl_t ssl(ctx);
+    ssl.open("www.google.com", proto);
+    printf("open %d\n", ssl.is_open());
 
-	ssl.writes("GET /\r\n\r\n");
+	ssl.putline("GET /\r\n\r\n");
     ssl.flush();
 
     char buf[256];
